#!/bin/bash

# Script for auto updating the helium miner.

# Set default values
MINER=miner
REGION=$(cat /root/.func/miner_region)
GWPORT=1680
MINERPORT=44158
DATADIR=/root/miner_data
LOGDIR=
QUAY_URL='https://quay.io/api/v1/repository/team-helium/miner/tag/?limit=50&page=1&onlyActiveTags=true'

echo $REGION
command -v jq > /dev/null || sudo apt-get install jq curl -y


# Read switches to override any default values for non-standard configs
while getopts n:g:p:d:l:r: flag
do
   case "${flag}" in
      n) MINER=${OPTARG};;
      g) GWPORT=${OPTARG};;
      p) MINERPORT=${OPTARG};;
      d) DATADIR=${OPTARG};;
      l) LOGDIR=${OPTARG};;
      r) REGION=${OPTARG};;
      *) echo "Exiting"; exit;;
   esac
done

# Autodetect running image version and set arch
if [ "$(uname -m)" == "x86_64" ]; then
        ARCH=amd
else
        ARCH=arm
fi

running_image=$(docker container inspect -f '{{.Config.Image}}' "$MINER" | awk -F: '{print $2}')


n=1
lastaddr=""
while true
do
addr=$(cat /root/.func/update/server_addr|head -$n|tail -1)
if [ "$addr" != "$lastaddr" ]
then
        lastaddr=$addr;
else
        echo "all server can not connection"
	echo "no version getted"
        exit
fi
version_api_url="http://$addr/download/patch/gateway_version"

echo $version_api_url

res=$(curl -H 'Cache-Control: no-cache'  -s --connect-timeout 10 --max-time 10 $version_api_url)
miner_latest=$(echo $res|jq .version|awk -F'"' '{print $2}')

if [ "$miner_latest" = "" ]
then
        n=`expr $n + 1`;
else
        break
fi

done
#miner_latest="gateway-v1.0.2"
echo $miner_latest


#echo "$0 starting with MINER=$MINER GWPORT=$GWPORT MINERPORT=$MINERPORT DATADIR=$DATADIR LOGDIR=$LOGDIR REGION=$REGION ARCH=$ARCH running_image=$running_image miner_latest=$miner_latest"

if [ "$miner_latest" = "$running_image" ];
then    echo "Miner image is already on the latest version"
        exit 0
else
         echo "find new version "$miner_latest
fi

# Pull the new miner image. Downloading it now will minimize miner downtime after stop.
echo "pull quay.io/team-helium/miner:"$miner_latest
docker pull quay.io/team-helium/miner:"$miner_latest"


res=$(docker images | grep "$miner_latest")
while [ "$res" = "" ];
do
        echo download new image failed;
        sleep 10
        docker pull quay.io/team-helium/miner:"$miner_latest"
        res=$(docker images | grep "$miner_latest")
done

if [ ! -d $DATADIR ];then mkdir $DATADIR;fi

echo "Stopping and removing old miner"
docker stop "$MINER" && docker rm "$MINER"

echo "Deleting old miner software"

for image in $(docker images | grep "miner" | awk '{print $3}'); do
	image_cleanup=$(docker images | grep "$image" | awk '{print $2}')
	#change this to $running_image if you want to keep the last 2 images
#	echo $image_cleanup $miner_latest
	if [ "$image_cleanup" = "$miner_latest" ]; then
	       continue
        else
		res=$(docker images |grep "$image" | grep "$image" -n | tail -1 | awk -F: '{print $1}')
		if [ "$res" != "1" ]
		then
			tmp1=$(docker images |grep "$image" | head -1 | awk '{print $1}')
			tmp2=$(docker images |grep "$image" | head -1 | awk '{print $2}')
			echo $tmp1:$tmp2
			docker rmi $tmp1:$tmp2
		fi

		echo "Cleaning up: $image_cleanup"
		docker image rm "$image"
        fi
done

killall get_miner_conf.sh
/root/.func/get_miner_conf.sh
echo "Provisioning new miner version"

if [ -n "$LOGDIR" ];
then
	LOGMOUNT="--mount type=bind,source=$LOGDIR,target=/var/data/log"
fi

#docker run -d --init --cpuset-cpus="0-3" --cpus 4 --ulimit nofile=200000:200000 --env REGION_OVERRIDE="$REGION" --restart always --publish "$GWPORT":"$GWPORT"/udp --publish "$MINERPORT":"$MINERPORT"/tcp --name "$MINER" --device /dev/i2c-1 --net host --privileged -v /var/run/dbus:/var/run/dbus --mount type=bind,source="$DATADIR",target=/var/data --mount type=bind,source=/etc/helium_gateway,target=/etc/helium_gateway quay.io/team-helium/miner:"$miner_latest"

docker run -d --init \
--ulimit nofile=64000:64000 \
--publish 127.0.0.1:1680:1680/udp \
--name miner \
--restart always \
--device /dev/i2c-1 \
--net host \
--privileged \
-v /var/run/dbus:/var/run/dbus \
--mount type=bind,source=/etc/helium_gateway/settings.toml,target=/etc/helium_gateway/settings.toml quay.io/team-helium/miner:"$miner_latest"
 
echo "miner image updated to $miner_latest"


